<?php
/**
 * This is the entrypoint for the locrMAPS for WooCommerce plugin.
 *
 * @package locrMAPS for WooCommerce
 */

declare(strict_types=1);

namespace Locr\WordPress\Plugin\LocrMapsForWooCommerce\DDD\ValueObjects;

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly.
}

/**
 * This class is responsible for the options.
 *
 * @property-read IconsQuerySelector $icons_query_selector The icons query selector.
 * @property-read SizesQuerySelector $sizes_query_selector The sizes query selector.
 */
final class Options {
	/**
	 * The sizes query selector.
	 *
	 * @var IconsQuerySelector
	 */
	private IconsQuerySelector $icons_query_selector;
	/**
	 * The sizes query selector.
	 *
	 * @var SizesQuerySelector
	 */
	private SizesQuerySelector $sizes_query_selector;

	/**
	 * The constructor.
	 *
	 * @param array<mixed> $shortcode_attributes The shortcode attributes.
	 * @param string       $only_option The only option.
	 */
	public function __construct( array $shortcode_attributes = array(), private string $only_option = '' ) {
		$this->icons_query_selector = new IconsQuerySelector();
		$this->sizes_query_selector = new SizesQuerySelector();

		$this->parse_options();
		$this->parse_attributes( $shortcode_attributes );
	}

	/**
	 * The magick getter.
	 *
	 * @param string $name The name of the property.
	 * @throws \InvalidArgumentException If the property does not exist.
	 */
	public function __get( string $name ): mixed {
		return match ( $name ) {
			'icons_query_selector' => $this->icons_query_selector,
			'sizes_query_selector' => $this->sizes_query_selector,
			default => throw new \InvalidArgumentException( "Property $name does not exist" ), // phpcs:ignore
		};
	}

	/**
	 * This function parses the attributes.
	 *
	 * @param array<mixed> $shortcode_attributes The shortcode attributes.
	 */
	private function parse_attributes( array $shortcode_attributes = array() ): void {
		if ( ( empty( $this->only_option ) || IconsQuerySelector::OPTION_KEY === $this->only_option ) && isset( $shortcode_attributes[ IconsQuerySelector::ATTRIBUTE_KEY ] ) && is_string( $shortcode_attributes[ IconsQuerySelector::ATTRIBUTE_KEY ] ) ) {
			$this->icons_query_selector = new IconsQuerySelector( trim( $shortcode_attributes[ IconsQuerySelector::ATTRIBUTE_KEY ] ) );
		}
		if ( ( empty( $this->only_option ) || SizesQuerySelector::OPTION_KEY === $this->only_option ) && isset( $shortcode_attributes[ SizesQuerySelector::ATTRIBUTE_KEY ] ) && is_string( $shortcode_attributes[ SizesQuerySelector::ATTRIBUTE_KEY ] ) ) {
			$this->sizes_query_selector = new SizesQuerySelector( trim( $shortcode_attributes[ SizesQuerySelector::ATTRIBUTE_KEY ] ) );
		}
	}

	/**
	 * This function parses the options.
	 */
	private function parse_options(): void {
		$options = get_option( 'locrmaps_for_woocommerce_options' );
		if ( ! is_array( $options ) ) {
			return;
		}

		if ( ( empty( $this->only_option ) || IconsQuerySelector::OPTION_KEY === $this->only_option ) && isset( $options[ IconsQuerySelector::OPTION_KEY ] ) && is_string( $options[ IconsQuerySelector::OPTION_KEY ] ) ) {
			$this->icons_query_selector = new IconsQuerySelector( trim( $options[ IconsQuerySelector::OPTION_KEY ] ) );
		}
		if ( ( empty( $this->only_option ) || SizesQuerySelector::OPTION_KEY === $this->only_option ) && isset( $options[ SizesQuerySelector::OPTION_KEY ] ) && is_string( $options[ SizesQuerySelector::OPTION_KEY ] ) ) {
			$this->sizes_query_selector = new SizesQuerySelector( trim( $options[ SizesQuerySelector::OPTION_KEY ] ) );
		}
	}
}
